@@ -24,23 +24,22 @@ mhd_malloc_word32(mutils_word32 size)
 unsigned char*
 mhd_digest(size_t id, const char* file)
 {
+    int            result = 1;
+    MHASH          hash   = MHASH_FAILED;
     FILE*          handle = NULL;
-    MHASH          hash   = NULL;
     unsigned char* buffer = NULL;
     size_t         size   = 0;
 
-    hash = mhash_init(id);
-    if (hash == MHASH_FAILED) {
+    if ((hash = mhash_init(id)) == MHASH_FAILED) {
         fprintf(stderr, "Could not mhash_init\n");
-        return NULL;
+        goto ERROR;
     }
- 
+
     /* file */
     if (file) {
         if (! (handle = fopen(file, "rb"))) {
             fprintf(stderr, "Could not open [%s]\n", file);
-            mhash_end(hash);
-            return NULL;
+            goto ERROR;
         }
     }
     /* stdin */
@@ -50,29 +49,46 @@ mhd_digest(size_t id, const char* file)
 
     if (! (buffer = malloc(MHD_FILE_BUFFER_SIZE)))  {
         fprintf(stderr, "Could not malloc\n");
-        mhash_end(hash);
-        fclose(handle);
-        return NULL;
+        goto ERROR;
     }
 
     while ((size = fread(buffer, 1, MHD_FILE_BUFFER_SIZE, handle)) > 0) {
         mhash(hash, buffer, size);
     }
+    goto EXIT;
 
-    free(buffer);
-    fclose(handle);
-    return mhash_end_m(hash, mhd_malloc_word32);
+ERROR:
+    result = 0;
+
+EXIT:
+    if (buffer) {
+        free(buffer);
+    }
+
+    if (file && handle) {
+        fclose(handle);
+    }
+
+    if (result) {
+        return mhash_end_m(hash, mhd_malloc_word32);
+    } else {
+        if (hash != MHASH_FAILED) {
+            mhash_end(hash);
+        }
+        return NULL;
+    }
 }
 
 /* Print a digest */
 int
 mhd_print(const char* name, size_t id, const char* file)
 {
-    unsigned char* d = NULL;
-    int            i = 0;
+    int            result = 1;
+    unsigned char* d      = NULL;
+    int            i      = 0;
 
     if (! (d = mhd_digest(id, file))) {
-        return 0;
+        goto ERROR;
     }
 
     if (file) {
@@ -82,9 +98,17 @@ mhd_print(const char* name, size_t id, const char* file)
         printf("%.2x", d[i]);
     }
     printf("\n");
+    goto EXIT;
 
-    free(d);
-    return 1;
+ERROR:
+    result = 0;
+
+EXIT:
+    if (d) {
+        free(d);
+    }
+
+    return result;
 }
 
 /* Trim (note that this function modify string) */
@@ -147,9 +171,10 @@ mhd_parse_bsd(char* buffer, char* name, char* file, char* dgst)
 int
 mhd_parse_gnu(char* buffer, char* name, char* file, char* dgst)
 {
-    size_t size = 0;
-    char*  sp   = NULL;
-    int    ok   = 0;
+    size_t      size = 0;
+    char*       sp   = NULL;
+    const char* n    = NULL;
+    int         ok   = 0;
 
     size = strlen(buffer);
     sp   = strchr(buffer, ' ');
@@ -163,21 +188,22 @@ mhd_parse_gnu(char* buffer, char* name, char* file, char* dgst)
         sp++;
     }
 
-    strncpy(dgst, mhd_trim(buffer), MHD_TEXT_BUFFER_SIZE);
-    strncpy(file, mhd_trim(sp),     MHD_TEXT_BUFFER_SIZE);
-
-    size = strlen(dgst);
+    size = strlen(buffer);
     switch (size) {
         case 32:
-            strncpy(name, "MD5", MHD_TEXT_BUFFER_SIZE);
+            n = "MD5";
             break;
         case 40:
-            strncpy(name, "SHA1", MHD_TEXT_BUFFER_SIZE);
+            n = "SHA1";
             break;
         default:
             return 0;
     }
 
+    strncpy(name, n,                MHD_TEXT_BUFFER_SIZE);
+    strncpy(file, mhd_trim(sp),     MHD_TEXT_BUFFER_SIZE);
+    strncpy(dgst, mhd_trim(buffer), MHD_TEXT_BUFFER_SIZE);
+
     return 1;
 }
 
@@ -226,7 +252,7 @@ mhd_command_check(const char* check_file)
     if (check_file) {
         if (! (handle = fopen(check_file, "rb"))) {
             fprintf(stderr, "Could not open [%s]\n", check_file);
-            return 1;
+            goto ERROR;
         }
     }
     /* stdin */
@@ -241,12 +267,7 @@ mhd_command_check(const char* check_file)
 
     if (! (buffer && name && file && dgst)) {
         fprintf(stderr, "Could not malloc\n");
-        free(buffer);
-        free(name);
-        free(file);
-        free(dgst);
-        fclose(handle);
-        return 1;
+        goto ERROR;
     }
 
     while (! feof(handle)) {
@@ -293,12 +314,31 @@ mhd_command_check(const char* check_file)
 
         printf("%s %s\n", (pass_each ? "PASS" : "FAIL"), file);
     }
+    goto EXIT;
 
-    free(buffer);
-    free(name);
-    free(file);
-    free(dgst);
-    fclose(handle);
+ERROR:
+    pass = 0;
+
+EXIT:
+    if (buffer) {
+        free(buffer);
+    }
+
+    if (name) {
+        free(name);
+    }
+
+    if (file) {
+        free(file);
+    }
+
+    if (dgst) {
+        free(dgst);
+    }
+
+    if (check_file && handle) {
+        fclose(handle);
+    }
 
     return ! pass;
 }
@@ -338,21 +378,17 @@ main(int ac, char** av)
     char*        name       = "SHA512";
     int          c          = -1;
     int          check      = 0;
-    int          bench      = 0;
     const char*  file       = NULL;
     char* const* file_first = NULL;
     char* const* file_last  = NULL;
 
     opterr = 1;
 
-    while ((c = getopt(ac, av, "a:bchl")) != -1) {
+    while ((c = getopt(ac, av, "a:chl")) != -1) {
         switch (c) {
             case 'a':
                 name = optarg;
                 break;
-            case 'b':
-                bench = 1;
-                break;
             case 'c':
                 check = 1;
                 break;
